# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-24 06:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djmessenger', '0008_auto_20160824_0519'),
    ]

    operations = [
        migrations.AddField(
            model_name='fbuserprofile',
            name='thumbups',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='first_name',
            field=models.CharField(blank=True, max_length=512, null=True, verbose_name='first name'),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='gender',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='last_name',
            field=models.CharField(blank=True, max_length=512, null=True, verbose_name='last name'),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='locale',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='profile_pic',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='psid',
            field=models.CharField(max_length=512, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='fbuserprofile',
            name='timezone',
            field=models.SmallIntegerField(),
        ),
        migrations.AlterField(
            model_name='messaging',
            name='body',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='messaging',
            name='type',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='persistentmenu',
            name='type',
            field=models.CharField(choices=[('postback', 'postback'), ('web_url', 'web_url')], default='web_url', help_text='postback means when clicking on the menu, it will send the payload back to the server; while web_url simply opens the url', max_length=10),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='latitude',
            field=models.DecimalField(blank=True, decimal_places=8, max_digits=10, null=True),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='longitude',
            field=models.DecimalField(blank=True, decimal_places=8, max_digits=11, null=True),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='mid',
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='seq',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='timestamp',
            field=models.BigIntegerField(help_text='Facebook returns timestamp as EPOCH'),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='url',
            field=models.URLField(blank=True, max_length=1000, null=True),
        ),
        migrations.AlterField(
            model_name='userlocation',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djmessenger.FBUserProfile'),
        ),
    ]
