# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-23 23:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djmessenger', '0005_auto_20160823_2249'),
    ]

    operations = [
        migrations.CreateModel(
            name='Messaging',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('body', models.TextField()),
                ('type', models.CharField(blank=True, max_length=128, null=True)),
            ],
        ),
        migrations.DeleteModel(
            name='RawBody',
        ),
    ]
