# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-24 06:49
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djmessenger', '0009_auto_20160824_0625'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='getstartedbutton',
            table='djm_getstarted',
        ),
    ]
