# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-23 22:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djmessenger', '0002_callback_fbuserprofile_getstartedbutton_persistentmenu'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserLocation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('latitude', models.DecimalField(blank=True, decimal_places=8, max_digits=10, null=True)),
                ('longitude', models.DecimalField(blank=True, decimal_places=8, max_digits=11, null=True)),
                ('timestamp', models.BigIntegerField(help_text='Facebook returns timestamp as EPOCH')),
                ('url', models.URLField(blank=True, max_length=1000, null=True)),
                ('mid', models.CharField(blank=True, max_length=512, null=True)),
                ('seq', models.IntegerField(blank=True, null=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djmessenger.FBUserProfile')),
            ],
            options={
                'get_latest_by': 'date_created',
            },
        ),
    ]
